/*	$Id: stereo.c 2009/11/16 17:59:51 mallet $ */

/* 
 * Copyright (c) 2007,2009 IS/AIST-ST2I/CNRS
 *	Joint Japanese-French Robotics Laboratory (JRL).
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <portLib.h>

#include <stdlib.h>
#include <h2timeLib.h>
#include "server/hueblobHeader.h"

#include "libstereopixel.h"
#include "shared.h"


/* --- hb_correlation_compute ---------------------------------------- */

STATUS
hb_correlation_compute(unsigned char *images[2], int width, int height,
		       double baseline, double intrinsic[9],
		       DATA_IM3D *im3d, int *report)
{
  double rectification[9];
  double baselines[2];
  SP_IMAGES_SIZES images_sizes;
  SP_CORRELATION_PARAMS correl_params;
  SP_IMAGES_TMP images_tmp[2];
  SP_CORRELATION_RESULTS correl_result;
  int s;

  baselines[1] = baseline;

  s = init_images_sizes(&images_sizes,
			height, width, 1, 1, 1, 0.0, 1.0, 0.0, 1.0);
  if (s != STEREO_OK) { *report = S_hueblob_STEREO; return ERROR; }

  s = init_images_tmp(&images_tmp[0], &images_sizes);
  if (s != STEREO_OK) { *report = S_hueblob_STEREO; return ERROR; }
  s = init_images_tmp(&images_tmp[1], &images_sizes);
  if (s != STEREO_OK) { *report = S_hueblob_STEREO; return ERROR; }

  s = init_correlation_params(&correl_params,
			      SP_CORRELATION_ZNCC,
			      SP_INTERPOLATION_DISP_PARABOLIQUE_ONLY,
			      9, 9,
			      -240, 0,
			      1.0, 0.0, 0.0, 0.0, 1);

  s = init_correlation_results(&images_sizes, &correl_params,
			       &correl_result);
  if (s != STEREO_OK) { *report = S_hueblob_STEREO; return ERROR; }

  s = moyennes_sigmas(images[0], &images_tmp[0], &correl_params);
  if (s != STEREO_OK) { *report = S_hueblob_STEREO; return ERROR; }
  s = moyennes_sigmas(images[1], &images_tmp[1], &correl_params);
  if (s != STEREO_OK) { *report = S_hueblob_STEREO; return ERROR; }

  s = spix_correl_zncc(images[0], images[1],
		       &images_tmp[0], &images_tmp[1],
		       &correl_params, &correl_result);
  if (s != STEREO_OK) { *report = S_hueblob_STEREO; return ERROR; }

  s = blob_filter_disparites(&correl_result, 100, 1.0);
  if (s != STEREO_OK) { *report = S_hueblob_STEREO; return ERROR; }

  s = reconstruction_3d(&correl_result,
			&images_tmp[0].images_sizes,
			SP_REPERE_RECTIFIE, SP_RECONSTRUCTION_NO_SIGMA,
			rectification, baselines,
			NULL, intrinsic, im3d);
  if (s != STEREO_OK) { *report = S_hueblob_STEREO; return ERROR; }

  sc_clean_images_tmp(&images_tmp[0]);
  sc_clean_images_tmp(&images_tmp[1]);
  sc_clean_correlation_results(&correl_result);
  return OK;
}
