/*	$Id: hueblobStruct.h 2011/06/24 14:41:35 mallet $ */

/*
 * Copyright (c) 2006,2009,2011 Anthony Mallet                  --  Wed Oct  4 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#if !defined HUEBLOB_STRUCT_H_
#define HUEBLOB_STRUCT_H_

/* max static strings length */
#ifndef HUEBLOB_MAX_STRLEN
#define HUEBLOB_MAX_STRLEN	256
#endif

typedef enum HueBlobBool {
  HUEBLOB_DISABLE,
  HUEBLOB_ENABLE
} HueBlobBool;

typedef struct HueBlobObject {
  char object[HUEBLOB_MAX_STRLEN];	/* object name */
  char file[HUEBLOB_MAX_STRLEN];	/* file name */
  double anchor_x;			/* 3D offset added to the 3D center of */
  double anchor_y;			/* the object, in "main" frame */
  double anchor_z;
} HueBlobObject;

typedef struct HueBlobInput {
  char bank[HUEBLOB_MAX_STRLEN];
  char camera[HUEBLOB_MAX_STRLEN];
  HueBlobBool use3d;
} HueBlobInput;


/* --- blob coordinates ---------------------------------------------- */

typedef struct HueBlobImageBox {
  char object[HUEBLOB_MAX_STRLEN];	/* object name */
  int nIter;
  int x, y;
  int width, height;
} HueBlobImageBox;

typedef struct HueBlob3DPoint {
  double x, y, z;
} HueBlob3DPoint;

typedef struct HueBlob3DBox {
  HueBlob3DPoint center;
  HueBlob3DPoint min, max;
} HueBlob3DBox;

typedef struct HueBlobBox {
  HueBlobImageBox video;
  HueBlob3DBox space;
  HueBlob3DBox absolute_space;
} HueBlobBox;


/* --- tracking requests --------------------------------------------- */

typedef struct HueBlobTrackBlob {
  char bank[HUEBLOB_MAX_STRLEN];	/**< bank with a stereo pair */
  char object[HUEBLOB_MAX_STRLEN];	/**< object name */
  int x, y, width, height;		/**< initial location hint */
  char name[HUEBLOB_MAX_STRLEN];	/** output poster name */
  HueBlobBool feedback;			/**< visual feedback */
} HueBlobTrackBlob;

typedef struct HueBlobTrackChessboard {
  char bank[HUEBLOB_MAX_STRLEN];	/**< bank with a stereo pair */
  int cols, rows;			/**< chessboard dimensions */
  char name[HUEBLOB_MAX_STRLEN];	/** output poster name */
  HueBlobBool feedback;			/**< visual feedback */
} HueBlobTrackChessboard;

#endif

